@@ -1,4 +1,4 @@
-# Copyright (C) 2017 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2017-2020 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -21,7 +21,7 @@ Show 'Did you know?' dialog at start up
 """
 
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2017, Damon Lynch"
+__copyright__ = "Copyright 2017-2020, Damon Lynch"
 
 
 from PyQt5.QtCore import pyqtSlot, QSize, Qt, QSettings, QUrl
@@ -33,18 +33,16 @@ from PyQt5.QtWidgets import (
     QDialogButtonBox, QTextBrowser
 )
 
-from gettext import gettext as _
-
 import raphodo.qrc_resources as qrc_resources
 from raphodo.preferences import Preferences
-from raphodo.viewutils import translateButtons
+from raphodo.viewutils import translateDialogBoxButtons
 
 tips = (
     (
         _(
             "Click on a file's checkbox to mark or unmark it for download."
         ),
-        'marksingle.png',
+        ':/tips/marksingle.png',
     ),
     (
         _(
@@ -53,7 +51,7 @@ tips = (
             "unchecked by default, and their thumbnails are dimmed so you can "
             "differentiate them from files that are yet to be downloaded."
         ),
-        'previouslydownloaded.png'
+        ':/tips/previouslydownloaded.png'
     ),
     (
         _(
@@ -61,14 +59,14 @@ tips = (
             "checkbox was clicked, regardless of whether they previously had a "
             "checkmark or not."
         ),
-        'markmany.png'
+        ':/tips/markmany.png'
     ),
     (
         _(
             "Click on a device's checkbox to quickly mark or unmark all its files for "
             "downloading."
         ),
-        'markall.png'
+        ':/tips/markall.png'
     ),
     (
         _(
@@ -76,7 +74,7 @@ tips = (
             "color indicates whether the download was successful (green), had file renaming "
             "problems (yellow/orange), or failed (red)."
         ),
-        'downloaded.png'
+        ':/tips/downloaded.png'
     ),
     (
         _(
@@ -86,7 +84,7 @@ tips = (
             window.
             """
         ),
-        'errorreporticon.png',
+        ':/tips/errorreporticon.png',
         _(
             """
             The Error Report window lists any problems encountered before, during or after the 
@@ -96,14 +94,14 @@ tips = (
             search box in the lower left of the Error Report window.
             """
         ),
-        'errorreport.png',
+        ':/tips/errorreport.png',
     ),
     (
         _(
             "You can simultaneously download from multiple cameras, smartphones, memory cards, "
             "and hard drives&mdash;as many devices as your computer can handle at one time."
         ),
-        'multipledevices.png'
+        ':/tips/multipledevices.png'
     ),
     (
         _(
@@ -112,7 +110,7 @@ tips = (
             "consecutive shots. Use it to identify photos and videos taken at different periods "
             "in a single day or over consecutive days."
         ),
-        'timeline.png',
+        ':/tips/timeline.png',
         _(
             """
 <p>In the illustration above, the first row of the Timeline is black because all the files on 
@@ -121,7 +119,7 @@ that date had been previously downloaded.</p>
 the Timeline:</p>
             """
         ),
-        'timelineadjust.png'
+        ':/tips/timelineadjust.png'
     ),
     (
         _(
@@ -130,7 +128,7 @@ the Timeline:</p>
             "When a time range is selected, the Timeline button on the left side of the main "
             "window will be highlighted."
         ),
-        'timelineselect.png',
+        ':/tips/timelineselect.png',
         _(
             "A download always includes all files that are checked for download, including those "
             "that are not currently displayed because the Timeline is being used."
@@ -142,7 +140,7 @@ the Timeline:</p>
             "You can hide or display the download sources by clicking on the name of "
             "the device you're downloading from at the top left of the program window."
         ),
-        'deviceshidden.png'
+        ':/tips/deviceshidden.png'
     ),
     (
       _(
@@ -159,7 +157,7 @@ instance.</li>
 <li><b>Device:</b> name of the device the photos and videos are being downloaded from.</li>
 </ol> """
         ),
-        'thumbnailsort.png'
+        ':/tips/thumbnailsort.png'
     ),
     (
         _(
@@ -167,7 +165,7 @@ instance.</li>
             "generate download subfolders and rename files as it downloads, using a scheme of your "
             "choosing."
         ),
-        'downloadwhereandrename.png',
+        ':/tips/downloadwhereandrename.png',
         _(
             "To specify where you want your files downloaded and how you want them named, open the "
             "appropriate panel on the right-side of the application window: "
@@ -191,7 +189,7 @@ They will be generated under the destination folder.</li>
 </ol>
             """
         ),
-        'defaultdownloaddirectory.png',
+        ':/tips/defaultdownloaddirectory.png',
         _(
             """
 You can download photos and videos to the same destination folder, or specify a different 
@@ -207,7 +205,7 @@ type.
             "generated subfolders if need be. A common scheme is to create a year "
             "subfolder and then a series of year-month-day subfolders within it."
         ),
-        'downloadsubfolders.png',
+        ':/tips/downloadsubfolders.png',
 
     ),
     (
@@ -223,7 +221,7 @@ Whenever possible, the program previews the download subfolders of photos and vi
 </ol>
             """
         ),
-        'downloadsubfolders.png',
+        ':/tips/downloadsubfolders.png',
     ),
     (
         _(
@@ -238,10 +236,10 @@ name of an event or location.</li>
 <li><b>Text</b> which you want to appear every time, such as a hyphen or a space.</li>
 </ol>
 Naming subfolders with the year, followed by the month and finally the day in numeric format makes 
-it easy to keep them sorted in a file manager, which is why its the default option:
+it easy to keep them sorted in a file manager, which is why it's the default option:
             """
         ),
-        'downloadsubfolders.png',
+        ':/tips/downloadsubfolders.png',
     ),
     (
         _(
@@ -251,14 +249,14 @@ you can use one of Rapid Photo Downloader's built-in presets, or create a custom
 the gear icon to bring up a drop-down menu:            
             """
         ),
-        'subfoldermenu.png',
+        ':/tips/subfoldermenu.png',
         _(
             """
 Using the drop-down menu, select a built-in preset or click on <b>Custom</b> to configure your own 
 scheme. You create your own schemes using the Photo or Video Subfolder Generation Editor:             
             """
         ),
-        'subfoldergeneration.png',
+        ':/tips/subfoldergeneration.png',
 
     ),
     (
@@ -267,7 +265,7 @@ scheme. You create your own schemes using the Photo or Video Subfolder Generatio
             "Simply use the <b>Filename Extension</b> as part of your download subfolder "
             "generation scheme:"
         ),
-        'subfoldergenerationext.png',
+        ':/tips/subfoldergenerationext.png',
         _('This illustration shows a saved custom preset named &quot;My custom preset&quot;.'),
     ),
     (
@@ -278,7 +276,7 @@ the generation of download subfolders that contain only the date the photos were
 Job Code:
             """
         ),
-        'subfoldergeneration.png'
+        ':/tips/subfoldergeneration.png'
     ),
     (
         _(
@@ -289,7 +287,7 @@ generate new combinations. Supposing you wanted a date format that combines year
 and month (MM) to form YYYY-MM. You can create it like this (note the red circle around the hyphen):
             """
         ),
-        'customdate.png',
+        ':/tips/customdate.png',
         _(
             """
 Read more about all the ways you can generate download subfolder names and file names in the <a 
@@ -306,7 +304,7 @@ use them in subfolder and file names. In this illustration, some files have had
 &quot;Green Bazaar&quot;: 
 """
         ),
-        'jobcodes.png',
+        ':/tips/jobcodes.png',
         _(
             """
 You can apply new or existing Job Codes before you start a download. If there are any 
@@ -319,7 +317,7 @@ a Job Code for them before the download begins.
         _(
             "Look for hints to guide you when working with Job Codes:"
         ),
-        'jobcodehint.png',
+        ':/tips/jobcodehint.png',
         _(
             "Hints will vary depending on the context, such as when the mouse is hovering over a "
             "button."
@@ -333,7 +331,7 @@ which file is which. Using <b>sequence numbers</b> to make filenames unique is h
 recommended!
             """
         ),
-        'photoeditordefault.png',
+        ':/tips/photoeditordefault.png',
         _(
             """
 <p>Four types of sequence values are available to help you assign unique names to your photos and 
@@ -358,7 +356,7 @@ The <b>Rename</b> panel allows you to configure file renaming. To rename your fi
 from among existing renaming presets or define your own.              
             """
         ),
-        'renameoptions.png',
+        ':/tips/renameoptions.png',
         _(
             """
 <p>The <b>Synchronize RAW + JPEG</b> option is useful if you use the RAW + JPEG feature on your 
@@ -380,7 +378,7 @@ external hard drives or network shares. Backup devices can be automatically dete
 backup locations specified.
             """
         ),
-        'backup.png',
+        ':/tips/backup.png',
         _(
             "In this example, the drive <b>photobackup</b> does not contain a folder named "
             "<tt>Videos</tt>, so videos will not be backed up to it."
@@ -394,7 +392,7 @@ sources, destinations, and backups. Additionally, settings can be reset to their
 default state, and caches and remembered files cleared.            
             """
         ) + _("You can also import program preferences from the older 0.4 version."),
-        'commandline.png'
+        ':/tips/commandline.png'
     ),
     (
         _(
@@ -441,7 +439,7 @@ class Tips:
                 else:
                     text = '{}{}<p></p>'.format(text, value)
             else:
-                text = '{}<img src=":/tips/{}">'.format(text, value)
+                text = '{}<img src="{}">'.format(text, value)
         return text
 
     def __len__(self):
@@ -466,7 +464,7 @@ class DidYouKnowDialog(QDialog):
 
         title = QLabel(_('Did you know...?'))
         title.setFont(titleFont)
-        pixmap = QIcon(':/did-you-know.svg').pixmap(QSize(pixsize, pixsize))  # type: QPixmap
+        pixmap = QIcon(':/tips/did-you-know.svg').pixmap(QSize(pixsize, pixsize))  # type: QPixmap
 
         icon = QLabel()
         icon.setPixmap(pixmap)
@@ -499,7 +497,7 @@ class DidYouKnowDialog(QDialog):
         self.previousButton = QPushButton(_('&Previous'))
 
         buttons = QDialogButtonBox(QDialogButtonBox.Close)
-        translateButtons(buttons)
+        translateDialogBoxButtons(buttons)
         buttons.addButton(self.previousButton, QDialogButtonBox.ActionRole)
         buttons.addButton(self.nextButton, QDialogButtonBox.ActionRole)
         self.previousButton.clicked.connect(self.previousButtonClicked)
