@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright (C) 2017 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2017-2020 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -23,7 +23,7 @@ Error log window for Rapid Photo Downloader
 """
 
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2017, Damon Lynch"
+__copyright__ = "Copyright 2017-2020, Damon Lynch"
 
 import logging
 import shlex
@@ -31,7 +31,7 @@ import subprocess
 import math
 from collections import deque, namedtuple
 from typing import Optional
-from gettext import gettext as _
+
 import re
 from html import escape
 
@@ -49,7 +49,7 @@ import raphodo.qrc_resources as qrc_resources
 from raphodo.constants import ErrorType
 from raphodo.rpdfile import RPDFile
 from raphodo.problemnotification import Problem, Problems
-from raphodo.viewutils import translateButtons
+from raphodo.viewutils import translateDialogBoxButtons
 from raphodo.storage import open_in_file_manager
 
 # ErrorLogMessage = namedtuple('ErrorLogMessage', 'title body name uri')
@@ -211,6 +211,10 @@ class ErrorReport(QDialog):
             QFontMetrics(QFont()).boundingRect(_('%s of %s matches') % (1000, 1000)).width()
             + spacing
         )
+        # Translators: match number of total matches in a search, e.g. 1 of 10 matches
+        _('%(matchnumber)s of %(total)s matches')
+
+        #TODO implement this once translations done
 
         findLayout = QHBoxLayout()
         findLayout.setSpacing(0)
@@ -229,7 +233,7 @@ class ErrorReport(QDialog):
 
 
         buttons = QDialogButtonBox(QDialogButtonBox.Close)
-        translateButtons(buttons)
+        translateDialogBoxButtons(buttons)
         self.clear = buttons.addButton(_('Clear'), QDialogButtonBox.ActionRole)  # type: QPushButton
         buttons.rejected.connect(self.reject)
         self.clear.clicked.connect(self.clearClicked)
@@ -245,9 +249,9 @@ class ErrorReport(QDialog):
         self.onFindChanged('')
 
         self.icon_lookup = {
-            ErrorType.warning: ':/report-warning.png',
-            ErrorType.serious_error: ':/report-error.png',
-            ErrorType.critical_error: ':/report-critical.png'
+            ErrorType.warning: ':/report/warning.svg',
+            ErrorType.serious_error: ':/report/error.svg',
+            ErrorType.critical_error: ':/report/critical.svg'
         }
 
     @pyqtSlot()
@@ -538,8 +542,10 @@ class SpeechBubble(QLabel):
         self.custom_height = max(math.ceil(QFontMetrics(self.counterFont).height() * 1.7), 24)
         self.counterPen = QPen(QColor(Qt.white))
         self.setStyleSheet("QLabel {border: 0px;}")
-        self.click_tooltip = _('The number of new entries added to the Error Report since it was '
-                               'last open. Click to open the Error Report.')
+        self.click_tooltip = _(
+            'The number of new entries added to the Error Report since it was '
+            'last open. Click to open the Error Report.'
+        )
 
     @property
     def count(self) -> int:
