@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright (C) 2011-2018 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2011-2020 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -19,7 +19,7 @@
 # see <http://www.gnu.org/licenses/>.
 
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2011-2018, Damon Lynch"
+__copyright__ = "Copyright 2011-2020, Damon Lynch"
 
 import logging
 import re
@@ -30,7 +30,6 @@ from typing import List, Tuple, Optional
 
 from PyQt5.QtCore import QSettings, QTime, Qt
 
-from gettext import gettext as _
 
 from raphodo.storage import (
     xdg_photos_directory, xdg_videos_directory, xdg_photos_identifier, xdg_videos_identifier
@@ -272,6 +271,7 @@ class Preferences:
     """
     Program preferences, being a mix of user facing and non-user facing prefs.
     """
+
     program_defaults = dict(program_version='')
     rename_defaults = dict(
         photo_download_folder=xdg_photos_directory(),
@@ -316,7 +316,10 @@ class Preferences:
         # see constants.MarkRawJpeg:
         mark_raw_jpeg=3,
         # introduced in 0.9.6b1:
-        auto_scroll=True
+        auto_scroll=True,
+        # If you change the language setting update it in __init__.py too, where it is
+        # read directly without using this class.
+        language='',
     )
     device_defaults = dict(
         only_external_mounts=True,
@@ -327,7 +330,7 @@ class Preferences:
         # pre 0.9.3a1 value: device_without_dcim_autodetection=False, is now replaced by
         # scan_specific_folders
         folders_to_scan=['DCIM', 'PRIVATE', 'MP_ROOT'],
-        ignored_paths=['.Trash', '.thumbnails'],
+        ignored_paths=['.Trash', '.thumbnails', 'THMBNL', '__MACOSX'],
         use_re_ignored_paths=False,
         volume_whitelist=[''],
         volume_blacklist=[''],
@@ -359,7 +362,7 @@ class Preferences:
     )
     error_defaults = dict(
         conflict_resolution=int(constants.ConflictResolution.skip),
-        backup_duplicate_overwrite=False
+        backup_duplicate_overwrite=False,
     )
     destinations = dict(
         photo_backup_destinations=[''],
@@ -374,7 +377,9 @@ class Preferences:
         purge_thumbnails=False,
         optimize_thumbnail_db=False
     )
-
+    metadata_defaults = dict(
+        force_exiftool=False,
+    )
 
     def __init__(self) -> None:
         # To avoid infinite recursions arising from the use of __setattr__,
@@ -383,15 +388,20 @@ class Preferences:
         self.__dict__['valid'] = True
 
         # These next two values must be kept in sync
-        dicts = (self.program_defaults, self.rename_defaults,
-                 self.timeline_defaults, self.display_defaults,
-                 self.device_defaults,
-                 self.backup_defaults, self.automation_defaults,
-                 self.performance_defaults, self.error_defaults,
-                 self.destinations, self.version_check, self.restart_directives)
-        group_names = ('Program', 'Rename', 'Timeline', 'Display', 'Device', 'Backup',
-                       'Automation', 'Performance', 'ErrorHandling', 'Destinations',
-                       'VersionCheck', 'RestartDirectives')
+        dicts = (
+            self.program_defaults, self.rename_defaults,
+            self.timeline_defaults, self.display_defaults,
+            self.device_defaults,
+            self.backup_defaults, self.automation_defaults,
+            self.performance_defaults, self.error_defaults,
+            self.destinations, self.version_check, self.restart_directives,
+            self.metadata_defaults,
+        )
+        group_names = (
+            'Program', 'Rename', 'Timeline', 'Display', 'Device', 'Backup',
+            'Automation', 'Performance', 'ErrorHandling', 'Destinations',
+            'VersionCheck', 'RestartDirectives', 'Metadata'
+        )
         assert len(dicts) == len(group_names)
 
         # Create quick lookup table for types of each value, including the
@@ -602,7 +612,7 @@ class Preferences:
         3. have two separators in a row
 
         :return: tuple with two values: (1) bool and error message if
-         prefs are invalid (else empy string)
+         prefs are invalid (else empty string)
         """
 
         msg = ''
@@ -774,6 +784,22 @@ class Preferences:
                 return True
         return False
 
+    def file_type_uses_job_code(self, file_type: FileType) -> bool:
+        """
+        Returns True if either the subfolder generation or file
+        renaming for the file type uses a Job Code.
+        """
+
+        if file_type == FileType.photo:
+            pref_lists = self.photo_rename, self.photo_subfolder
+        else:
+            pref_lists = self.video_rename, self.video_subfolder
+
+        for pref_list in pref_lists:
+            if self.pref_uses_job_code(pref_list):
+                return True
+        return False
+
     def most_recent_job_code(self, missing: Optional[str]=None) -> str:
         """
         Get the most recent Job Code used (which is assumed to be at the top).
@@ -944,8 +970,9 @@ class Preferences:
                 self.settings.endGroup()
                 default = self.defaults['conflict_resolution']
                 default_name = constants.ConflictResolution(default).name
-                logging.warning('Resetting Conflict Resolution preference value to %s',
-                                default_name)
+                logging.warning(
+                    'Resetting Conflict Resolution preference value to %s', default_name
+                )
                 self.conflict_resolution = default
             # destinationButtonPressed is no longer used by 0.9.0a5
             self.settings.beginGroup("MainWindow")
@@ -954,7 +981,7 @@ class Preferences:
                 if self.settings.contains(key):
                     logging.debug("Removing preference value %s", key)
                     self.settings.remove(key)
-            except:
+            except Exception:
                 logging.warning("Unknown error removing %s preference value", key)
             self.settings.endGroup()
 
@@ -1004,6 +1031,18 @@ class Preferences:
                     "scan_specific_folders because it doesn't exist"
                 )
 
+        v0919b2 = pkg_resources.parse_version('0.9.19b2')
+        key = 'ignored_paths'
+        group = 'device_defaults'
+        if previous_version < v0919b2 and self.value_is_set(key, group):
+            # Versions prior to 0.9.19b2 did not include all the ignored paths
+            # introduced in 0.9.16 and 0.9.19b2. If the user already has some
+            # values, these new defaults will not be added automatically. So add
+            # them here.
+            for value in ('THMBNL', '__MACOSX'):
+                # If the value is not already in the list, add it
+                self.add_list_value(key=key, value=value)
+
 
     def validate_max_CPU_cores(self) -> None:
         logging.debug('Validating CPU core count for thumbnail generation...')
@@ -1027,6 +1066,12 @@ class Preferences:
 
         return ext.upper() not in self.ignore_unhandled_file_exts
 
+    def settings_path(self) -> str:
+        """
+        :return: the full path of the settings file
+        """
+        return self.settings.fileName()
+
 
 def match_pref_list(pref_lists: List[List[str]], user_pref_list: List[str]) -> int:
     try:
