@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2017 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2011-2020 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -17,7 +17,7 @@
 # see <http://www.gnu.org/licenses/>.
 
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2011-2017, Damon Lynch"
+__copyright__ = "Copyright 2011-2020, Damon Lynch"
 
 import os
 import time
@@ -26,95 +26,27 @@ import uuid
 import logging
 import mimetypes
 from collections import Counter, UserDict
-from urllib.request import pathname2url
 import locale
-from collections import defaultdict
-from typing import Optional, List, Tuple, Union, Any, Dict
+from typing import Optional, List, Tuple, Union, Any
 
-from gettext import gettext as _
 import gi
+
 gi.require_version('GLib', '2.0')
 from gi.repository import GLib
 
 import raphodo.exiftool as exiftool
-from raphodo.constants import (DownloadStatus, FileType, FileExtension, FileSortPriority,
-                               ThumbnailCacheStatus, Downloaded, Desktop, thumbnail_offset,
-                               DeviceTimestampTZ, ThumbnailCacheDiskStatus, ExifSource)
+from raphodo.constants import (
+    DownloadStatus, FileType, FileExtension, FileSortPriority, ThumbnailCacheStatus, Downloaded,
+    DeviceTimestampTZ, ThumbnailCacheDiskStatus, ExifSource,
+)
 
 from raphodo.storage import get_uri, CameraDetails
 import raphodo.metadataphoto as metadataphoto
 import raphodo.metadatavideo as metadatavideo
+import raphodo.metadataexiftool as metadataexiftool
 from raphodo.utilities import thousands, make_internationalized_list, datetime_roughly_equal
 from raphodo.problemnotification import Problem, make_href
-
-
-RAW_EXTENSIONS = [
-    '3fr', 'arw', 'dcr', 'cr2', 'crw',  'dng', 'mos', 'mef', 'mrw', 'nef', 'nrw', 'orf', 'pef',
-    'raf', 'raw', 'rw2', 'sr2', 'srw'
-]
-
-JPEG_EXTENSIONS = ['jpg', 'jpe', 'jpeg']
-
-JPEG_TYPE_EXTENSIONS = ['jpg', 'jpe', 'jpeg', 'mpo']
-
-OTHER_PHOTO_EXTENSIONS = ['tif', 'tiff', 'mpo']
-
-NON_RAW_IMAGE_EXTENSIONS = JPEG_EXTENSIONS + OTHER_PHOTO_EXTENSIONS
-
-PHOTO_EXTENSIONS = RAW_EXTENSIONS + NON_RAW_IMAGE_EXTENSIONS
-
-PHOTO_EXTENSIONS_WITHOUT_OTHER = RAW_EXTENSIONS + JPEG_EXTENSIONS
-
-PHOTO_EXTENSIONS_SCAN = PHOTO_EXTENSIONS
-
-AUDIO_EXTENSIONS = ['wav', 'mp3']
-
-
-VIDEO_EXTENSIONS = [
-    '3gp', 'avi', 'm2t', 'm2ts', 'mov', 'mp4', 'mpeg','mpg', 'mod', 'tod', 'mts'
-]
-
-VIDEO_THUMBNAIL_EXTENSIONS = ['thm']
-
-ALL_USER_VISIBLE_EXTENSIONS = PHOTO_EXTENSIONS + VIDEO_EXTENSIONS + ['xmp', 'log']
-
-ALL_KNOWN_EXTENSIONS = ALL_USER_VISIBLE_EXTENSIONS + AUDIO_EXTENSIONS + VIDEO_THUMBNAIL_EXTENSIONS
-
-MUST_CACHE_VIDEOS = [video for video in VIDEO_EXTENSIONS if thumbnail_offset.get(video) is None]
-
-
-def file_type(file_extension: str) -> Optional[FileType]:
-    """
-    Returns file type (photo/video), or None if it's neither.
-    Checks only the file's extension
-    """
-
-    if file_extension in PHOTO_EXTENSIONS_SCAN:
-        return FileType.photo
-    elif file_extension in VIDEO_EXTENSIONS:
-        return FileType.video
-    return None
-
-
-def extension_type(file_extension: str) -> FileExtension:
-    """
-    Returns the type of file as indicated by the filename extension.
-
-    :param file_extension: lowercase filename extension
-    :return: Enum indicating file type
-    """
-    if file_extension in RAW_EXTENSIONS:
-        return FileExtension.raw
-    elif file_extension in JPEG_EXTENSIONS:
-        return FileExtension.jpeg
-    elif file_extension in OTHER_PHOTO_EXTENSIONS:
-        return FileExtension.other_photo
-    elif file_extension in VIDEO_EXTENSIONS:
-        return FileExtension.video
-    elif file_extension in AUDIO_EXTENSIONS:
-        return FileExtension.audio
-    else:
-        return FileExtension.unknown
+import raphodo.fileformats as fileformats
 
 
 def get_sort_priority(extension: FileExtension, file_type: FileType) -> FileSortPriority:
@@ -158,7 +90,6 @@ def get_rpdfile(name: str,
                 raw_exif_bytes: Optional[bytes],
                 exif_source: Optional[ExifSource],
                 problem: Optional[Problem]):
-
     if file_type == FileType.video:
         return Video(
             name=name,
@@ -213,27 +144,27 @@ def get_rpdfile(name: str,
 
 
 def file_types_by_number(no_photos: int, no_videos: int) -> str:
-        """
-        Generate a string show number of photos and videos
+    """
+    Generate a string show number of photos and videos
 
-        :param no_photos: number of photos
-        :param no_videos: number of videos
-        """
-        if (no_videos > 0) and (no_photos > 0):
-            v = _('photos and videos')
-        elif (no_videos == 0) and (no_photos == 0):
-            v = _('photos or videos')
-        elif no_videos > 0:
-            if no_videos > 1:
-                v = _('videos')
-            else:
-                v = _('video')
+    :param no_photos: number of photos
+    :param no_videos: number of videos
+    """
+    if (no_videos > 0) and (no_photos > 0):
+        v = _('photos and videos')
+    elif (no_videos == 0) and (no_photos == 0):
+        v = _('photos or videos')
+    elif no_videos > 0:
+        if no_videos > 1:
+            v = _('videos')
         else:
-            if no_photos > 1:
-                v = _('photos')
-            else:
-                v = _('photo')
-        return v
+            v = _('video')
+    else:
+        if no_photos > 1:
+            v = _('photos')
+        else:
+            v = _('photo')
+    return v
 
 
 def make_key(file_t: FileType, path: str) -> str:
@@ -242,11 +173,12 @@ def make_key(file_t: FileType, path: str) -> str:
 
 class FileSizeSum(UserDict):
     """ Sum size in bytes of photos and videos """
+
     def __missing__(self, key):
         self[key] = 0
         return self[key]
 
-    def sum(self, basedir: Optional[str]=None) -> int:
+    def sum(self, basedir: Optional[str] = None) -> int:
         if basedir is not None:
             return self[make_key(FileType.photo, basedir)] + self[make_key(FileType.video, basedir)]
         else:
@@ -317,6 +249,9 @@ class FileTypeCounter(Counter):
          present e.g. "photos and videos"
         """
         file_types_present = self.file_types_present()
+        # Translators: %(variable)s represents Python code, not a plural of the term
+        # variable. You must keep the %(variable)s untranslated, or the program will
+        # crash.
         file_count_summary = _("%(number)s %(filetypes)s") % dict(
             number=thousands(self[FileType.photo] + self[FileType.video]),
             filetypes=file_types_present
@@ -325,8 +260,9 @@ class FileTypeCounter(Counter):
 
     def file_types_present_details(self, title_case=True, singular_natural=False) -> str:
         """
+        Displays details about how many files are selected or ready to be downloaded.
 
-        :param title_case:
+        :param title_case: whether the details should use title case or not.
         :param singular_natural: if True, instead of '1 photo', return 'A photo'. If True,
          title_case parameter is treated as always False.
         :return:
@@ -336,15 +272,21 @@ class FileTypeCounter(Counter):
         v = self[FileType.video]
 
         if v > 1:
-            videos =  _('%(no_videos)s Videos') % dict(no_videos=thousands(v))
+            # Translators: %(variable)s represents Python code, not a plural of the term
+            # variable. You must keep the %(variable)s untranslated, or the program will
+            # crash.
+            videos = _('%(no_videos)s Videos') % dict(no_videos=thousands(v))
         elif v == 1:
             if singular_natural:
                 # translators: natural language expression signifying a single video
                 videos = _('a video')
             else:
-                videos =  _('1 Video')
+                videos = _('1 Video')
 
         if p > 1:
+            # Translators: %(variable)s represents Python code, not a plural of the term
+            # variable. You must keep the %(variable)s untranslated, or the program will
+            # crash.
             photos = _('%(no_photos)s Photos') % dict(no_photos=thousands(p))
         elif p == 1:
             if singular_natural:
@@ -393,12 +335,12 @@ class RPDFile:
                  from_camera: bool,
                  never_read_mdatatime: bool,
                  device_display_name: str,
-                 device_uri:str,
-                 camera_details: Optional[CameraDetails]=None,
-                 camera_memory_card_identifiers: Optional[List[int]]=None,
-                 raw_exif_bytes: Optional[bytes]=None,
-                 exif_source: Optional[ExifSource]=None,
-                 problem: Optional[Problem]=None) -> None:
+                 device_uri: str,
+                 camera_details: Optional[CameraDetails] = None,
+                 camera_memory_card_identifiers: Optional[List[int]] = None,
+                 raw_exif_bytes: Optional[bytes] = None,
+                 exif_source: Optional[ExifSource] = None,
+                 problem: Optional[Problem] = None) -> None:
         """
 
         :param name: filename, including the extension, without its path
@@ -476,9 +418,9 @@ class RPDFile:
         self._assign_file_type()
 
         # Remove the period from the extension and make it lower case
-        self.extension = os.path.splitext(name)[1][1:].lower()
+        self.extension = fileformats.extract_extension(name)
         # Classify file based on its type e.g. jpeg, raw or tiff etc.
-        self.extension_type = extension_type(self.extension)
+        self.extension_type = fileformats.extension_type(self.extension)
 
         self.mime_type = mimetypes.guess_type(name)[0]
 
@@ -495,7 +437,7 @@ class RPDFile:
         # If None, haven't tried yet
         # If False, no problems encountered, got it (or it was assigned from mtime
         # when never_read_mdatatime is True)
-        self._no_datetime_metadata = None  #type: Optional[bool]
+        self._no_datetime_metadata = None  # type: Optional[bool]
 
         self.never_read_mdatatime = never_read_mdatatime
         if never_read_mdatatime:
@@ -504,7 +446,7 @@ class RPDFile:
         self.device_timestamp_type = device_timestamp_type
 
         ###########
-        #self.ctime
+        # self.ctime
         ###########
         #
         # self.ctime is the photo or video's creation time. It's value depends
@@ -594,8 +536,8 @@ class RPDFile:
         self.download_subfolder = ''
         self.download_path = ''  # os.path.join(download_folder, download_subfolder)
         self.download_name = ''
-        self.download_full_file_name = '' # filename with path
-        self.download_full_base_name = '' # filename with path but no extension
+        self.download_full_file_name = ''  # filename with path
+        self.download_full_base_name = ''  # filename with path but no extension
         self.download_thm_full_name = ''  # name of THM (thumbnail) file with path
         self.download_xmp_full_name = ''  # name of XMP sidecar with path
         self.download_log_full_name = ''  # name of LOG associate file with path
@@ -606,8 +548,8 @@ class RPDFile:
         self.xmp_extension = ''
         self.log_extension = ''
 
-        self.metadata = None # type: Optional[Union[metadataphoto.MetaData, metadatavideo.MetaData]]
-        self.metadata_failure = False # type: bool
+        self.metadata = None  # type: Optional[Union[metadataphoto.MetaData, metadatavideo.MetaData, metadataexiftool.MetadataExiftool]]
+        self.metadata_failure = False  # type: bool
 
         # User preference values used for name generation
         self.subfolder_pref_list = []  # type: List[str]
@@ -631,7 +573,7 @@ class RPDFile:
         return self._mtime
 
     @modification_time.setter
-    def modification_time(self, value: Union[float, int])  -> None:
+    def modification_time(self, value: Union[float, int]) -> None:
         """
         See notes on self.ctime above
         """
@@ -681,7 +623,7 @@ class RPDFile:
 
         return not datetime_roughly_equal(self._mdatatime, self._mtime)
 
-    def date_time(self, missing: Optional[Any]=None) -> datetime:
+    def date_time(self, missing: Optional[Any] = None) -> datetime:
         """
         Returns the date time as found in the file's metadata, and caches it
         for later use.
@@ -714,7 +656,7 @@ class RPDFile:
         self.mdatatime = self._datetime.timestamp()
         return self._datetime
 
-    def timestamp(self, missing: Optional[Any]=None) -> float:
+    def timestamp(self, missing: Optional[Any] = None) -> float:
         """
         Returns the time stamp as found in the file's metadata, and
         caches it for later use.
@@ -727,7 +669,6 @@ class RPDFile:
         :return: the metadata's date time value, else missing if not found or error
         """
 
-
         dt = self.date_time(missing=missing)
         if self._no_datetime_metadata:
             return missing
@@ -741,7 +682,6 @@ class RPDFile:
         """
         return self.mime_type == 'image/jpeg'
 
-
     def is_jpeg_type(self) -> bool:
         """
         :return:True if the image is a jpeg or MPO image
@@ -760,7 +700,14 @@ class RPDFile:
 
         :return: True if the image is a RAW file
         """
-        return self.extension in RAW_EXTENSIONS
+        return self.extension in fileformats.RAW_EXTENSIONS
+
+    def is_heif(self) -> bool:
+        """
+        Inspects file extension to determine if an HEIF / HEIC file
+        :return:
+        """
+        return self.extension in fileformats.HEIF_EXTENTIONS
 
     def is_tiff(self) -> bool:
         """
@@ -775,18 +722,39 @@ class RPDFile:
         return self.audio_file_full_name is not None
 
     def get_current_full_file_name(self) -> str:
+        """
+        :return: full file name which depending on download status will be the
+         source file or the destination file
+        """
+
         if self.status in Downloaded:
             return self.download_full_file_name
         else:
             return self.full_file_name
 
+    def get_current_sample_full_file_name(self) -> str:
+        """
+        Sample files can be temporary extracts on the file system, or source
+        or destination files on the file system
+
+        :return: full file name assuming the current file is a sample file.
+        """
+
+        # take advantage of Python's left to right evaluation:
+        return self.temp_sample_full_file_name or self.get_current_full_file_name()
+
     def get_current_name(self) -> str:
+        """
+        :return: file name which depending on download status will be the
+         source file or the destination file
+        """
+
         if self.status in Downloaded:
             return self.download_name
         else:
             return self.name
 
-    def get_uri(self, desktop_environment: Optional[bool]=True) -> str:
+    def get_uri(self, desktop_environment: Optional[bool] = True) -> str:
         """
         Generate and return the URI for the file
 
@@ -797,12 +765,15 @@ class RPDFile:
         """
 
         if self.status in Downloaded:
-            return 'file://{}'.format(pathname2url(self.download_full_file_name))
+            path = self.download_full_file_name
+            camera_details = None
         else:
-            return get_uri(
-                full_file_name = self.full_file_name, camera_details=self.camera_details,
-                desktop_environment=desktop_environment
-            )
+            path = self.full_file_name
+            camera_details = self.camera_details
+        return get_uri(
+            full_file_name=path, camera_details=camera_details,
+            desktop_environment=desktop_environment
+        )
 
     def get_souce_href(self) -> str:
         return make_href(
@@ -825,8 +796,11 @@ class RPDFile:
         """
 
         if self.from_camera:
-            return _('%(path)s on %(camera)s') % dict(path=self.full_file_name,
-                                                     camera=self.camera_display_name)
+            # Translators: %(variable)s represents Python code, not a plural of the term variable.
+            # You must keep the %(variable)s untranslated, or the program will crash.
+            return _('%(path)s on %(camera)s') % dict(
+                path=self.full_file_name, camera=self.camera_display_name
+            )
         else:
             return self.full_file_name
 
@@ -834,24 +808,26 @@ class RPDFile:
         self.file_type = None
 
     def __repr__(self):
-        return "{}\t{}".format(self.name, datetime.fromtimestamp(
-            self.modification_time).strftime('%Y-%m-%d %H:%M:%S'))
+        return "{}\t{}\t{}".format(
+            self.name, datetime.fromtimestamp(self.modification_time).strftime('%Y-%m-%d %H:%M:%S'),
+            self.get_current_sample_full_file_name()
+        )
 
 
 class Photo(RPDFile):
-
     title = _("photo")
     title_capitalized = _("Photo")
 
     def _assign_file_type(self):
         self.file_type = FileType.photo
 
-    def load_metadata(self, full_file_name: Optional[str]=None,
-                 raw_bytes: Optional[bytearray]=None,
-                 app1_segment: Optional[bytearray]=None,
-                 et_process: exiftool.ExifTool=None) -> bool:
+    def load_metadata(self, full_file_name: Optional[str] = None,
+                      raw_bytes: Optional[bytearray] = None,
+                      app1_segment: Optional[bytearray] = None,
+                      et_process: exiftool.ExifTool = None,
+                      force_exiftool: Optional[bool] = False) -> bool:
         """
-        Use GExiv2 to read the photograph's metadata.
+        Use GExiv2 or ExifTool to read the photograph's metadata.
 
         :param full_file_name: full path of file from which file to read
          the metadata.
@@ -860,34 +836,45 @@ class Photo(RPDFile):
         :param app1_segment: the app1 segment of a jpeg file, from which
          the metadata can be read
         :param et_process: optional daemon ExifTool process
+        :param force_exiftool: whether ExifTool must be used to load the
+         metadata
         :return: True if successful, False otherwise
         """
 
-        try:
-            self.metadata = metadataphoto.MetaData(full_file_name=full_file_name,
-               raw_bytes=raw_bytes, app1_segment=app1_segment, et_process=et_process)
-        except GLib.GError as e:
-            logging.warning("Could not read metadata from %s. %s", self.full_file_name, e)
-            self.metadata_failure = True
-            return False
-        except:
-            logging.warning("Could not read metadata from %s", self.full_file_name)
-            self.metadata_failure = True
-            return False
-        else:
+        if force_exiftool or fileformats.use_exiftool_on_photo(
+                self.extension, preview_extraction_irrelevant=True):
+
+            self.metadata = metadataexiftool.MetadataExiftool(
+                full_file_name=full_file_name, et_process=et_process, file_type=self.file_type
+            )
             return True
+        else:
+            try:
+                self.metadata = metadataphoto.MetaData(
+                    full_file_name=full_file_name, raw_bytes=raw_bytes,
+                    app1_segment=app1_segment, et_process=et_process,
+                )
+            except GLib.GError as e:
+                logging.warning("Could not read metadata from %s. %s", self.full_file_name, e)
+                self.metadata_failure = True
+                return False
+            except:
+                logging.warning("Could not read metadata from %s", self.full_file_name)
+                self.metadata_failure = True
+                return False
+            else:
+                return True
 
 
 class Video(RPDFile):
-
     title = _("video")
     title_capitalized = _("Video")
 
     def _assign_file_type(self):
         self.file_type = FileType.video
 
-    def load_metadata(self, full_file_name: Optional[str]=None,
-                 et_process: exiftool.ExifTool=None) -> bool:
+    def load_metadata(self, full_file_name: Optional[str] = None,
+                      et_process: exiftool.ExifTool = None) -> bool:
         """
         Use ExifTool to read the video's metadata
         :param full_file_name: full path of file from which file to read
